/**
 * Copyright (c) 2014-2024, MindFusion LLC - Bulgaria.
 */

import java.awt.*;
import java.io.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;

import com.mindfusion.drawing.*;
import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.components.*;
import com.mindfusion.diagramming.components.TextComponent;


public class OrgChartNode extends CompositeNode
{
	static Document xmlTemplate = loadXmlFile("OrgChartNode.xml");

	// empty constructor required for deserialization
	public OrgChartNode()
	{
	}
	
	public OrgChartNode(OrgChartNode prototype)
	{
		super(prototype);
		findComponents();
	}

	public OrgChartNode(Diagram diagram)
	{
		super(diagram);

		// create components from XML template
		getComponents().add(XmlLoader.load(xmlTemplate, null, null));
		findComponents();

		// customize appearance
		title.getTextFormat().setHorizontalAlign(Align.Near);
		fullName.getTextFormat().setHorizontalAlign(Align.Near);
		text.getTextFormat().setHorizontalAlign(Align.Near);
		shape.setIsOutline(true);
		setFill(new SolidBrush(Color.WHITE));
		setStroke(new Pen(Color.BLACK, 0.5));
	}
	
	@Override
	protected void onLoad(Diagram diagram)
	{
		super.onLoad(diagram);
		findComponents();
	}
	
	void findComponents()
	{
		// find components by name
		shape = (ShapeComponent) findComponent("Shape");
		image = (ImageComponent) findComponent("Image");
		title = (TextComponent) findComponent("Title");
		fullName = (TextComponent) findComponent("FullName");
		text = (TextComponent) findComponent("Text");
	}

	static Document loadXmlFile(String filepath)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringElementContentWhitespace(true);
		factory.setNamespaceAware(true);

		Document document = null;
		DocumentBuilder builder;
		try
		{
			File file = new File(filepath);
			builder = factory.newDocumentBuilder();
			document = builder.parse(file); 
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return document;
	}

	public Pen getStroke()
	{
	    return shape.getPen();
	}
	
	public void setStroke(Pen value)
	{
		shape.setPen(value);
	}
	
	public Brush getFill()
	{
	    return shape.getBrush();
	}
	
	public void setFill(Brush value)
	{
		shape.setBrush(value);
	}

	public Image getImage()
	{
		return image.getImage();   
	}
	
	public void setImage(Image value)
	{
		image.setImage(value);
	}

	public String getTitle()
	{
	    return title.getText();
	}
	
	public void setTitle(String value)
	{
		title.setText(value);
	}

	public String getFullName()
	{
	    return fullName.getText();
	}
	
	public void setFullName(String value)
	{
		fullName.setText(value);
	}

	public String getText()
	{
		return text.getText();
	}
	
	public void setText(String value)
	{
		text.setText(value);
	}
	
	// undo and redo support

	@Override
	protected DiagramItemProperties createProperties()
	{
		return new OrgChartNodeProperties();
	}

	@Override
	protected void saveProperties(DiagramItemProperties props)
	{
		super.saveProperties(props);

		OrgChartNodeProperties state = (OrgChartNodeProperties)props;
		state.Title = getTitle();
		state.FullName = getFullName();
		state.Text = getText();
		state.Stroke = getStroke();
		state.Fill = getFill();
		state.Image = getImage();
	}

	@Override
	protected void restoreProperties(DiagramItemProperties props)
	{
		super.restoreProperties(props);

		OrgChartNodeProperties state = (OrgChartNodeProperties)props;
		setTitle(state.Title);
		setFullName(state.FullName);
		setText(state.Text);
		setStroke(state.Stroke);
		setFill(state.Fill);
		setImage(state.Image);
	}
	
	private ShapeComponent shape;
	private ImageComponent image;
	private TextComponent title;
	private TextComponent fullName;
	private TextComponent text;
	
	static final long serialVersionUID = 1L;
}